﻿using System;
using Ozeki.Media.MediaHandlers;
using Ozeki.Media;
using Ozeki.VoIP;
using Ozeki.VoIP.SDK;

namespace CommandLineCaller
{
    class Program
    {
        #region Fields
        static string userName;
        static string registerName;
        static string registerPassword;
        static string displayName;
        static int domainServerPort;
        static string domainServerHost;
        static string mp3FilePath;
        static string dialNumber;
        static bool registrationRequired;

        static MP3StreamPlayback mp3StreamPlayback;
        static PhoneCallMediaSender phoneCallMediaSender;
        static MediaConnector mediaConnector;


        static IPhoneLine phoneLine;
        static ISoftPhone softPhone;
        static IPhoneCall phoneCall;
        static string localAddress;
        #endregion

        static void Main(string[] args)
        {
            //Egyes parancssori paraméterek alapértelmezett értékkel rendelkeznek
            #region Default values

            displayName = "CommandLineCaller";
            domainServerPort = 5060;
            registerPassword = string.Empty;
            registrationRequired = false;
            localAddress = SoftPhoneFactory.GetLocalIP().ToString();
            mp3FilePath = "..\\..\\Resources\\test.mp3";

            #endregion

            
            if(!ParseArgs(args))
            {
                //A paraméterek parsolása nem sikerült, ezért megjelenítjük a helpet.
                PrintHelp();
                Console.ReadLine();
                return;
            }

            try
            {
                //Try to reading mp3 file
                mp3StreamPlayback = new MP3StreamPlayback(mp3FilePath);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.ReadLine();
                return;
            }

            phoneCallMediaSender = new PhoneCallMediaSender();
            mediaConnector = new MediaConnector();

            mediaConnector.Connect(mp3StreamPlayback, phoneCallMediaSender);

            // Creates a SoftPhone
            softPhone = SoftPhoneFactory.CreateSoftPhone(localAddress, 5000, 5100, 5060);

            // Create a phone line using the softphone.
            phoneLine = softPhone.CreatePhoneLine(new SIPAccount(registrationRequired, displayName, userName, registerName,
                                                         registerPassword, domainServerHost, domainServerPort));

            // Create event handler for the phone line.
            phoneLine.PhoneLineStateChanged += phoneLine_PhoneLineStateChanged;

            softPhone.RegisterPhoneLine(phoneLine);

            Console.ReadLine();

        }

        /// <summary>
        /// A phoneLine állapota megváltozott
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        static void phoneLine_PhoneLineStateChanged(object sender, VoIPEventArgs<PhoneLineState> e)
        {
            Console.WriteLine("Phone line state has changed to {0}.", e.Item);

            //Ha a regisztráció sikeres vagy nincs szükség regisztrációra, akkor létrehozunk egy hívást
            if (e.Item == PhoneLineState.NoRegNeeded || e.Item == PhoneLineState.RegistrationSucceded)
                CreateCall();
        }

        /// <summary>
        /// Létrehozza és elindítja a hívást
        /// </summary>
        static void CreateCall()
        {
            Console.WriteLine("Creating new call ...");

            //Létrehozunk egy hívást
            phoneCall = softPhone.CreateCallObject(phoneLine, dialNumber);

            //Feliratkozunk az eseményeire
            phoneCall.CallErrorOccured += phoneCall_CallErrorOccured;
            phoneCall.CallStateChanged += phoneCall_CallStateChanged;

            //A phoneCallMediaSender hez hozzá attacholjuk a hívást
            phoneCallMediaSender.AttachToCall(phoneCall);

            //Elindítjuk a hívást
            phoneCall.Start();


        }

        /// <summary>
        /// A hívás állapota megváltozott
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        static void phoneCall_CallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            Console.WriteLine("Call state has changed to {0}", e.Item);

            //Ha a hívás in call állapotba kerül, akkor elkezdi bejátszani az mp3 file-t a hívásba
            if (e.Item == CallState.InCall)
                mp3StreamPlayback.StartStreaming();
        }

        /// <summary>
        /// A hívás közben hiba történt
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        static void phoneCall_CallErrorOccured(object sender, VoIPEventArgs<CallError> e)
        {
            Console.WriteLine("Call error has occurred. {0}", e.Item);
        }

        /// <summary>
        /// Parsolom a parancssori paramétereket
        /// </summary>
        /// <param name="args"></param>
        /// <returns>Ha true val tér vissza, akkor validak, ha false-al, akkor nem validak a paraméterek</returns>
        static bool ParseArgs(string[] args)
        {
            var isValid = true;

            for (int i = 0; i < args.Length; i++)
            {
                if (!isValid)
                    break;

                switch (args[i])
                {
                    case "-u": //username
                        if (i + 1 < args.Length)
                        {
                            userName = args[i + 1];
                            ++i;
                        }

                        else
                        {
                            Console.WriteLine("User name is not specified.");
                            isValid = false;
                        }
                        break;
                    case "-rr": //registration required
                        if (i + 1 < args.Length)
                        {
                            if (bool.TryParse(args[i + 1], out registrationRequired))
                            {
                                ++i;
                            }
                            else
                            {
                                Console.WriteLine("Registration required is not valid. {0}", args[i + 1]);
                                isValid = false;
                            }

                        }
                        else
                        {
                            Console.WriteLine("Registration required is not specified.");
                            isValid = false;
                        }
                        break;
                    case "-r": //register name
                        if (i + 1 < args.Length)
                        {
                            registerName = args[i + 1];
                            ++i;
                        }
                        else
                        {
                            Console.WriteLine("Register name is not specified.");
                            isValid = false;
                        }
                        break;
                    case "-d": //display name
                        if (i + 1 < args.Length)
                        {
                            displayName = args[i + 1];
                            ++i;
                        }

                        else
                        {
                            Console.WriteLine("Display name is not specified.");
                            isValid = false;
                        }
                        break;
                    case "-p": //password
                        if (i + 1 < args.Length)
                        {
                            registerPassword = args[i + 1];
                            ++i;
                        }
                        else
                        {
                            Console.WriteLine("Register password is not specified.");
                            isValid = false;
                        }
                        break;
                    case "-dialNumber": //dial number
                        if (i + 1 < args.Length)
                        {
                            dialNumber = args[i + 1];
                            ++i;
                        }
                        else
                        {
                            Console.WriteLine("Dial number is not specified.");
                            isValid = false;
                        }
                        break;
                    case "-host": //pbx host
                        if (i + 1 < args.Length)
                        {
                            domainServerHost = args[i + 1];
                            ++i;

                        }
                        else
                        {
                            Console.WriteLine("Host is not specified.");
                            isValid = false;
                        }
                        break;
                    case "-port": //pbx port
                        if (i + 1 < args.Length)
                        {
                            if (int.TryParse(args[i + 1], out domainServerPort))
                            {
                                if(domainServerPort < 0)
                                {
                                    Console.WriteLine("Port number must be greater or equal than 0.");
                                    isValid = false;
                                }

                                else if(domainServerPort > 65535)
                                {
                                    Console.WriteLine("Port number must be lesser or equal than 66535");
                                    isValid = false;
                                }
                                else
                                {
                                    ++i;
                                }
                            }
                            else
                            {
                                Console.WriteLine("Invalid port number: {0}", args[i + 1]);
                                isValid = false;
                            }

                        }
                        else
                        {
                            Console.WriteLine("Port is not specified.");
                            isValid = false;
                        }
                        break;
                    case "-path": //mp3 file path
                        if (i + 1 < args.Length)
                        {
                            mp3FilePath = args[i + 1];
                            ++i;
                        }
                        else
                        {
                            Console.WriteLine("MP3 file path is not specified.");
                            isValid = false;
                        }
                        break;
                    case "-localAddress": // local address
                        if (i + 1 < args.Length)
                        {
                            localAddress = args[i + 1];
                            ++i;
                        }
                        else
                        {
                            Console.WriteLine("Local address is not specified.");
                            isValid = false;
                        }
                        break;
                    default:
                        Console.WriteLine("Invalid argument {0}", args[i]);
                        isValid = false;
                        break;

                }
            }

            //Ha a paraméterek nem érvényesek, akkor false értékkel térünk vissza
            if (!isValid)
                return isValid;


            if (string.IsNullOrEmpty(userName) || string.IsNullOrEmpty(registerName) || string.IsNullOrEmpty(displayName) || string.IsNullOrEmpty(domainServerHost) || string.IsNullOrEmpty(mp3FilePath) || string.IsNullOrEmpty(dialNumber))
                isValid = false;

            return isValid;
        }

        /// <summary>
        /// Megjeleníti a képernyőn a parancssori paramétereket
        /// </summary>
        static void PrintHelp()
        {
            Console.WriteLine("Usage: -u oz890 -r oz890 -p oz890 -dialNumber 873 -host 192.168.115.1");
            Console.WriteLine("Options: ");
            Console.WriteLine("-rr - SIP account registration is required (default value: false)");
            Console.WriteLine("-u - SIP account username");
            Console.WriteLine("-r - SIP account register name");
            Console.WriteLine("-d - SIP account display name (default value: CommandLineCaller)");
            Console.WriteLine("-p - SIP account password (default value: [empty string])");
            Console.WriteLine("-host - PBX address");
            Console.WriteLine("-port - PBX port (default value: 5060");
            Console.WriteLine("-path - MP3 file path (default value: ..\\..\\Resources\\test.mp3");

            Console.WriteLine("-localAddress - Your local address: (default value: {0})", SoftPhoneFactory.GetLocalIP());
            Console.WriteLine("-dialNumber - The dialed number");
        }
    }
}
